@@ -142,7 +142,7 @@ secondsFromTicks ::
    EventList.T NonNegW.Rational Event.T
 secondsFromTicks division =
    EventList.catMaybes .
-   flip MS.evalState MetaEvent.defltST .
+   flip MS.evalState MetaEvent.defltTempo .
    EventList.mapM
       (\ticks -> do
          microsPerQN <- MS.get
