@@ -1,19 +1,20 @@
 module Sound.MIDI.File.Event.Meta (
    T(..),
-   Key(..), Scale(..),
    ElapsedTime, fromElapsedTime, toElapsedTime,
    Tempo,       fromTempo,       toTempo,
+   defltTempo,
    SMPTEHours, SMPTEMinutes, SMPTESeconds, SMPTEFrames, SMPTEBits,
-   defltST, defltDurT,
    get, put, ) where
 
 import Sound.MIDI.Message.Channel (Channel, toChannel, fromChannel, )
 
-import           Sound.MIDI.Parser.Primitive
+import qualified Sound.MIDI.KeySignature as KeySig
+
+import Sound.MIDI.Parser.Primitive (get1, get2, get3, getVar, getBigN, )
 import qualified Sound.MIDI.Parser.Class as Parser
 import qualified Sound.MIDI.Parser.Restricted as ParserRestricted
 
-import Control.Monad (liftM, liftM2, liftM4, liftM5, )
+import Control.Monad (liftM, liftM4, liftM5, )
 
 import qualified Sound.MIDI.Writer.Basic as Writer
 import qualified Sound.MIDI.Bit as Bit
@@ -23,14 +24,11 @@ import Sound.MIDI.Monoid ((+#+))
 import qualified Numeric.NonNegative.Wrapper as NonNeg
 import Sound.MIDI.IO (ByteList, listCharFromByte, listByteFromChar, )
 
-import Data.Ix (Ix, index, )
 import Sound.MIDI.Utility
-         (arbitraryString, arbitraryByteList,
-          enumRandomR, boundedEnumRandom, chooseEnum, )
+         (arbitraryString, arbitraryByteList, )
 
 import Test.QuickCheck (Arbitrary(arbitrary), )
 import qualified Test.QuickCheck as QC
-import System.Random (Random(random, randomR), )
 
 import Prelude hiding (putStr, )
 
@@ -59,7 +57,7 @@ data T =
    | SetTempo Tempo
    | SMPTEOffset SMPTEHours SMPTEMinutes SMPTESeconds SMPTEFrames SMPTEBits
    | TimeSig Int Int Int Int
-   | KeySig Key Scale
+   | KeySig KeySig.T
    | SequencerSpecific ByteList
    | Unknown Int ByteList
      deriving (Show, Eq, Ord)
@@ -81,7 +79,7 @@ instance Arbitrary T where
          liftM  (SetTempo . NonNeg.fromNumberMsg "Tempo always positive") (QC.choose (0,0xFFFFFF)) :
          liftM5 SMPTEOffset arbitraryByte arbitraryByte arbitraryByte arbitraryByte arbitraryByte :
          liftM4 TimeSig arbitraryByte arbitraryByte arbitraryByte arbitraryByte :
-         liftM2 KeySig arbitrary arbitrary :
+         liftM  KeySig arbitrary :
          liftM  SequencerSpecific arbitraryByteList :
 --         liftM  Unknown arbitrary arbitraryByteList :
          []
@@ -90,57 +88,6 @@ arbitraryByte :: QC.Gen Int
 arbitraryByte = QC.choose (0,0xFF::Int)
 
 
-{- |
-The following enumerated type lists all the keys in order of their key
-signatures from flats to sharps.
-(@Cf@ = 7 flats, @Gf@ = 6 flats ... @F@ = 1 flat, @C@ = 0 flats\/sharps,
-@G@ = 1 sharp, ... @Cs@ = 7 sharps.)
-Useful for transposition.
--}
-data Key = KeyCf | KeyGf | KeyDf | KeyAf | KeyEf | KeyBf | KeyF
-         | KeyC  | KeyG  | KeyD  | KeyA  | KeyE  | KeyB  | KeyFs | KeyCs
-             deriving (Show, Eq, Ord, Ix, Enum, Bounded)
-
-
-instance Random Key where
-   random  = boundedEnumRandom
-   randomR = enumRandomR
-
-instance Arbitrary Key where
-   arbitrary = chooseEnum
-
-
-
-{- |
-The Key Signature specifies a mode, either major or minor.
--}
-data Scale = Major | Minor
-            deriving (Show, Eq, Ord, Ix, Enum, Bounded)
-
-
-instance Random Scale where
-   random  = boundedEnumRandom
-   randomR = enumRandomR
-
-instance Arbitrary Scale where
-   arbitrary = chooseEnum
-
-
-
-{- |
-Default number of quarter notes in a second.
--}
-defltDurT :: ElapsedTime
-defltDurT = 2
-
-{- |
-The default SetTempo value, in microseconds per quarter note.
-This expresses the default of 120 beats per minute.
--}
-defltST :: Tempo
-defltST = div 1000000 (fromIntegral defltDurT)
-
-
 toElapsedTime :: Integer -> ElapsedTime
 toElapsedTime = NonNeg.fromNumberMsg "toElapsedTime"
 
@@ -154,6 +101,13 @@ toTempo = NonNeg.fromNumberMsg "toTempo"
 fromTempo :: Tempo -> Int
 fromTempo = NonNeg.toNumber
 
+{- |
+The default SetTempo value, in microseconds per quarter note.
+This expresses the default of 120 beats per minute.
+-}
+defltTempo :: Tempo
+defltTempo = 500000
+
 
 
 -- * serialization
@@ -193,20 +147,12 @@ get =
                    b <- get1
                    return (TimeSig n d c b)
 
-         089 -> parse $
-                do
-                   sf <- get1
-                   sc <- getEnum
-                   return (KeySig (toKeyName sf) sc)
+         089 -> parse $ liftM KeySig KeySig.get
 
          127 -> liftM SequencerSpecific $ getBigN len
 
          _   -> liftM (Unknown code) $ getBigN len
 
-toKeyName :: Int -> Key
-toKeyName sf = toEnum (mod (sf+7) 15)
-
-
 
 put :: Writer.C writer => T -> writer
 put ev =
@@ -227,11 +173,7 @@ put ev =
      SMPTEOffset hr mn se fr ff
                       -> putList  84 [hr,mn,se,fr,ff]
      TimeSig n d c b  -> putList  88 [n,d,c,b]
-     KeySig sf mi     -> putList  89 [sf', fromEnum mi]
-                             where k = index (KeyCf,KeyCs) sf - 7
-                                   sf' = if k >= 0
-                                           then k
-                                           else 255+k
+     KeySig key       -> putList  89 $ KeySig.toBytes key
      SequencerSpecific codes
                       -> putByteList 127 codes
      Unknown typ s    -> putByteList typ s
