@@ -63,10 +63,13 @@ instance Monad T where
 fromIO :: (IO.Handle -> IO a) -> T a
 fromIO act = Cons $ lift . act =<< ask
 
+ioeTry :: IO a -> IO (Either IOError a)
+ioeTry = Exc.try
+
 fallibleFromIO :: (IO.Handle -> IO a) -> Parser.Fallible T a
 fallibleFromIO act =
    Sync.ExceptionalT . Cons . lift .
-      fmap (Sync.mapException show . Sync.fromEither) . IOE.try . act
+      fmap (Sync.mapException show . Sync.fromEither) . ioeTry . act
           =<< lift (Cons ask)
 
 instance Parser.EndCheck T where
